@@ -97,7 +97,7 @@ int esd_send_auth( int sock )
 	goto exit_fd;
 
     /* we've run the gauntlet, everything's ok, proceed as usual */
-    fsync( sock );
+    /* fsync( sock ); */
     retval = 1;
 
  exit_fd:
@@ -167,9 +167,9 @@ int esd_resume( int esd )
 
 /*******************************************************************/
 /* initialize the socket to send data to the sound daemon */
-int esd_open_sound( char *host )
+int esd_open_sound( const char *host )
 {
-    char *espeaker = NULL;
+    const char *espeaker = NULL;
     struct hostent *he;
 
     /*********************/
@@ -253,6 +253,7 @@ int esd_open_sound( char *host )
     {
 	fprintf(stderr,"Unable to connect to server port %d\n", 
 		socket_addr.sin_port );
+	fprintf(stderr,"This generally means that the program could not talk to the esound server\n");
 	return -1;
     }
 
@@ -268,7 +269,8 @@ int esd_open_sound( char *host )
 
 /*******************************************************************/
 /* open a socket for playing as a stream */
-int esd_play_stream( esd_format_t format, int rate, char *host, char *name )
+int esd_play_stream( esd_format_t format, int rate, 
+		     const char *host, const char *name )
 {
     int sock;
     int proto = ESD_PROTO_STREAM_PLAY;
@@ -295,15 +297,40 @@ int esd_play_stream( esd_format_t format, int rate, char *host, char *name )
     if( write( sock, name_buf, ESD_NAME_MAX ) != ESD_NAME_MAX )
 	return -1;
 
+  /* Reduce buffers on sockets to the minimum needed */
+    {
+      int fd_bufsize;
+      int src_format;
+      
+      src_format = format;
+      fd_bufsize = ESD_BUF_SIZE;
+      if (rate > 0)
+	fd_bufsize = (fd_bufsize * 44100) / rate;
+      else
+	fd_bufsize = (fd_bufsize * 44100) / 44100;
+      if ((src_format & ESD_MASK_BITS) == ESD_BITS16)
+	{
+	  fd_bufsize *= 2;
+	}
+      if (!((src_format & ESD_MASK_CHAN) == ESD_MONO))
+	{
+	  fd_bufsize *= 2;	      
+	}
+      setsockopt(sock, SOL_SOCKET, SO_SNDBUF, 
+		 &fd_bufsize, sizeof(fd_bufsize));
+      setsockopt(sock, SOL_SOCKET, SO_RCVBUF,  
+		 &fd_bufsize, sizeof(fd_bufsize));
+    }
     /* flush the socket */
-    fsync( sock );
+    /* fsync( sock ); */
     
     return sock;
 }
 
 /*******************************************************************/
 /* open a socket for playing as a stream, fallback to /dev/dsp */
-int esd_play_stream_fallback( esd_format_t format, int rate, char *host, char *name )
+int esd_play_stream_fallback( esd_format_t format, int rate, 
+			      const char *host, const char *name )
 {
     int socket_out;
     /* try to open a connection to the server */
@@ -326,7 +353,8 @@ int esd_play_stream_fallback( esd_format_t format, int rate, char *host, char *n
 
 /*******************************************************************/
 /* open a socket for monitoring as a stream */
-int esd_monitor_stream( esd_format_t format, int rate, char *host, char *name )
+int esd_monitor_stream( esd_format_t format, int rate, 
+			const char *host, const char *name )
 {
     int sock;
     int proto = ESD_PROTO_STREAM_MON;
@@ -353,15 +381,40 @@ int esd_monitor_stream( esd_format_t format, int rate, char *host, char *name )
     if( write( sock, name_buf, ESD_NAME_MAX ) != ESD_NAME_MAX )
 	return -1;
 
+  /* Reduce buffers on sockets to the minimum needed */
+    {
+      int fd_bufsize;
+      int src_format;
+      
+      src_format = format;
+      fd_bufsize = ESD_BUF_SIZE;
+      if (rate > 0)
+	fd_bufsize = (fd_bufsize * 44100) / rate;
+      else
+	fd_bufsize = (fd_bufsize * 44100) / 44100;
+      if ((src_format & ESD_MASK_BITS) == ESD_BITS16)
+	{
+	  fd_bufsize *= 2;
+	}
+      if (!((src_format & ESD_MASK_CHAN) == ESD_MONO))
+	{
+	  fd_bufsize *= 2;	      
+	}
+      setsockopt(sock, SOL_SOCKET, SO_SNDBUF, 
+		 &fd_bufsize, sizeof(fd_bufsize));
+      setsockopt(sock, SOL_SOCKET, SO_RCVBUF,  
+		 &fd_bufsize, sizeof(fd_bufsize));
+    }
     /* flush the socket */
-    fsync( sock );
+    /* fsync( sock ); */
     
     return sock;
 }
 
 /*******************************************************************/
 /* open a socket for filtering as a stream */
-int esd_filter_stream( esd_format_t format, int rate, char *host, char *name )
+int esd_filter_stream( esd_format_t format, int rate, 
+		       const char *host, const char *name )
 {
     int sock;
     int proto = ESD_PROTO_STREAM_FILT;
@@ -388,15 +441,40 @@ int esd_filter_stream( esd_format_t format, int rate, char *host, char *name )
     if( write( sock, name_buf, ESD_NAME_MAX ) != ESD_NAME_MAX )
 	return -1;
 
+  /* Reduce buffers on sockets to the minimum needed */
+    {
+      int fd_bufsize;
+      int src_format;
+      
+      src_format = format;
+      fd_bufsize = ESD_BUF_SIZE;
+      if (rate > 0)
+	fd_bufsize = (fd_bufsize * 44100) / rate;
+      else
+	fd_bufsize = (fd_bufsize * 44100) / 44100;
+      if ((src_format & ESD_MASK_BITS) == ESD_BITS16)
+	{
+	  fd_bufsize *= 2;
+	}
+      if (!((src_format & ESD_MASK_CHAN) == ESD_MONO))
+	{
+	  fd_bufsize *= 2;	      
+	}
+      setsockopt(sock, SOL_SOCKET, SO_SNDBUF, 
+		 &fd_bufsize, sizeof(fd_bufsize));
+      setsockopt(sock, SOL_SOCKET, SO_RCVBUF,  
+		 &fd_bufsize, sizeof(fd_bufsize));
+    }
     /* flush the socket */
-    fsync( sock );
+    /* fsync( sock ); */
     
     return sock;
 }
 
 /*******************************************************************/
 /* open a socket for recording as a stream */
-int esd_record_stream( esd_format_t format, int rate, char *host, char *name )
+int esd_record_stream( esd_format_t format, int rate, 
+		       const char *host, const char *name )
 {
     int sock;
     int proto = ESD_PROTO_STREAM_REC;
@@ -423,15 +501,40 @@ int esd_record_stream( esd_format_t format, int rate, char *host, char *name )
     if( write( sock, name_buf, ESD_NAME_MAX ) != ESD_NAME_MAX )
 	return -1;
 
+  /* Reduce buffers on sockets to the minimum needed */
+    {
+      int fd_bufsize;
+      int src_format;
+      
+      src_format = format;
+      fd_bufsize = ESD_BUF_SIZE;
+      if (rate > 0)
+	fd_bufsize = (fd_bufsize * 44100) / rate;
+      else
+	fd_bufsize = (fd_bufsize * 44100) / 44100;
+      if ((src_format & ESD_MASK_BITS) == ESD_BITS16)
+	{
+	  fd_bufsize *= 2;
+	}
+      if (!((src_format & ESD_MASK_CHAN) == ESD_MONO))
+	{
+	  fd_bufsize *= 2;	      
+	}
+      setsockopt(sock, SOL_SOCKET, SO_SNDBUF, 
+		 &fd_bufsize, sizeof(fd_bufsize));
+      setsockopt(sock, SOL_SOCKET, SO_RCVBUF,  
+		 &fd_bufsize, sizeof(fd_bufsize));
+    }
     /* flush the socket */
-    fsync( sock );
+    /* fsync( sock ); */
     
     return sock;
 }
 
 /*******************************************************************/
 /* open a socket for recording as a stream, fallback to /dev/dsp */
-int esd_record_stream_fallback( esd_format_t format, int rate, char *host, char *name )
+int esd_record_stream_fallback( esd_format_t format, int rate, 
+				const char *host, const char *name )
 {
     int socket_out;
 
@@ -448,6 +551,30 @@ int esd_record_stream_fallback( esd_format_t format, int rate, char *host, char
     esd_audio_format = format;
     esd_audio_rate = rate;
     socket_out = esd_audio_open();
+  /* Reduce buffers on sockets to the minimum needed */
+    {
+      int fd_bufsize;
+      int src_format;
+      
+      src_format = format;
+      fd_bufsize = ESD_BUF_SIZE;
+      if (rate > 0)
+	fd_bufsize = (fd_bufsize * 44100) / rate;
+      else
+	fd_bufsize = (fd_bufsize * 44100) / 44100;
+      if ((src_format & ESD_MASK_BITS) == ESD_BITS16)
+	{
+	  fd_bufsize *= 2;
+	}
+      if (!((src_format & ESD_MASK_CHAN) == ESD_MONO))
+	{
+	  fd_bufsize *= 2;	      
+	}
+      setsockopt(socket_out, SOL_SOCKET, SO_SNDBUF, 
+		 &fd_bufsize, sizeof(fd_bufsize));
+      setsockopt(socket_out, SOL_SOCKET, SO_RCVBUF,  
+		 &fd_bufsize, sizeof(fd_bufsize));
+    }
 
     /* we either got it, or we didn't */
     return socket_out;
@@ -455,7 +582,8 @@ int esd_record_stream_fallback( esd_format_t format, int rate, char *host, char
 
 /*******************************************************************/
 /* cache a sample in the server returns sample id, <= 0 is error */
-int esd_sample_cache( int esd, esd_format_t format, int rate, int size, char *name )
+int esd_sample_cache( int esd, esd_format_t format, int rate, 
+		      const int size, const char *name )
 {
     int id = 0;
     int proto = ESD_PROTO_SAMPLE_CACHE;
@@ -483,7 +611,7 @@ int esd_sample_cache( int esd, esd_format_t format, int rate, int size, char *na
 	return -1;
 
     /* flush the socket */
-    fsync( esd );
+    /* fsync( esd ); */
 
     /* get the sample id back from the server */
     if ( read( esd, &id, sizeof(id) ) != sizeof(id) )
@@ -529,7 +657,7 @@ int esd_sample_getid( int esd, const char *name)
 	return -1;
 
     /* flush the socket */
-    fsync( esd );
+    /* fsync( esd ); */
 
     /* get the sample id back from the server */
     if ( read( esd, &id, sizeof(id) ) != sizeof(id) )
@@ -553,7 +681,7 @@ int esd_sample_free( int esd, int sample )
 	return -1;
     if ( write( esd, &sample, sizeof(sample) ) != sizeof(sample) )
 	return -1;
-    fsync( esd );
+    /* fsync( esd ); */
 
     /* get the sample id back from the server */
     if ( read( esd, &id, sizeof(id) ) != sizeof(id) )
@@ -577,7 +705,7 @@ int esd_sample_play( int esd, int sample )
 	return -1;
     if ( write( esd, &sample, sizeof(sample) ) != sizeof(sample) )
 	return -1;
-    fsync( esd );
+    /* fsync( esd ); */
 
     /* get the sample id back from the server */
     if ( read( esd, &is_ok, sizeof(is_ok) ) != sizeof(is_ok) )
@@ -602,7 +730,7 @@ int esd_sample_loop( int esd, int sample )
 	return -1;
     if ( write( esd, &sample, sizeof(sample) ) != sizeof(sample) )
 	return -1;
-    fsync( esd );
+    /* fsync( esd ); */
 
     /* get the sample id back from the server */
     if ( read( esd, &is_ok, sizeof(is_ok) ) != sizeof(is_ok) )
@@ -626,7 +754,7 @@ int esd_sample_stop( int esd, int sample )
 	return -1;
     if ( write( esd, &sample, sizeof(sample) ) != sizeof(sample) )
 	return -1;
-    fsync( esd );
+    /* fsync( esd ); */
 
     /* get the sample id back from the server */
     if ( read( esd, &is_ok, sizeof(is_ok) ) != sizeof(is_ok) )
