@@ -626,9 +626,8 @@ int mix_players_16s( void *output, int length )
 	    if ( actual > max ) max = actual;
 	    
 	} else if ( actual == 0 ) {
-	    if ( esdbg_trace) 
-		printf( "no data available from player (%d)\n", 
-			iterator->source_id, iterator ); 
+	    ESDBG_TRACE( printf( "no data available from player (%d)\n", 
+				 iterator->source_id, iterator ); );
 	} else {
 	    /* actual < 0 means erase the player */
 	    erase = iterator;
@@ -644,9 +643,8 @@ int mix_players_16s( void *output, int length )
 	}
     }
 
-    if ( esdbg_trace ) {
-	printf( "maximum stream length = %d bytes\n", max );
-    }
+    /* if ( esdbg_comms ) printf( "maximum stream length = %d bytes\n", max ); */
+    ESDBG_COMMS( printf( "maximum stream length = %d bytes\n", max ); );
 
     if ( (esd_audio_format & ESD_MASK_BITS) == ESD_BITS16 ) 
 	clip_mix_to_output_16s( output, max );
