@@ -19,8 +19,11 @@ void clear_auth( int signum )
 {
     int i;
 
-    if ( esdbg_trace ) 
-	printf( "(ca) resetting ownership of sound daemon\n" );
+    /* NOTE: attempt to only include debugging info if desired. if this doesn't
+       work on your compiler, please email ericmit@ix.netcom.com with info */
+    /* if ( esdbg_trace ) 
+	printf( "(ca) resetting ownership of sound daemon\n" ); */
+    ESDBG_TRACE( printf( "(ca) resetting ownership of sound daemon\n" ); );
 
     /* reset the access rights */
     esd_is_owned = 0;
@@ -55,9 +58,9 @@ int check_endian( esd_client_t *client )
 	/* printf( "different endian order!\n" ); */
 	client->swap_byte_order = 1;
     } else {
-	if ( esdbg_trace ) 
+	ESDBG_TRACE( 
 	    printf( "(%02d) unknown endian key: 0x%08x (same = 0x%08x, diff = 0x%08x)\n",
-		    client->fd, endian, ESD_ENDIAN_KEY, ESD_SWAP_ENDIAN_KEY );
+		    client->fd, endian, ESD_ENDIAN_KEY, ESD_SWAP_ENDIAN_KEY ); );
 	
 	return -1;
     }
@@ -79,8 +82,8 @@ int validate_source( esd_client_t *client, struct sockaddr_in source, int owner_
 
     if( !esd_is_owned ) {
 	/* noone owns it yet, the first client claims ownership */
-	if ( esdbg_trace )
-	    printf( "(%02d) esd auth: claiming ownership of esd.\n", client->fd );
+	ESDBG_TRACE( printf( "(%02d) esd auth: claiming ownership of esd.\n", 
+			     client->fd ); );
 
 	esd_is_locked = 1;
 	memcpy( esd_owner_key, submitted_key, ESD_KEY_LEN );
@@ -103,16 +106,15 @@ int validate_source( esd_client_t *client, struct sockaddr_in source, int owner_
     /* TODO: maybe check based on source ip? */
     /* if ( !owner_only ) { check_ip_etc(); } */
     /* the client is not authorized to connect to the server */
-    if ( esdbg_trace )
-	printf( "(%02d) esd auth: NOT authorized to use esd, closing conn.\n",
-		client->fd );
+    ESDBG_TRACE( printf( "(%02d) esd auth: NOT authorized to use esd, closing conn.\n",
+			 client->fd ); );
 
     return 0;
 
  check_endian:
     if ( check_endian( client ) <= 0 ) { 
 	/* irix is weird, may return -1, instead of EAGAIN, try again anyway */
-	if ( esdbg_trace ) printf( "will check for endian key next trip\n" );
+	ESDBG_TRACE( printf( "will check for endian key next trip\n" ); );
 	client->state = ESD_NEEDS_ENDCHECK;
 	return 1;
     }
@@ -128,12 +130,11 @@ int esd_proto_lock( esd_client_t *client )
     int ok = 1, actual;		/* already validated, can't fail */
     int client_ok = maybe_swap_32( client->swap_byte_order, ok );
 
-    if ( esdbg_trace )
-	printf( "(%02d) locking sound daemon\n", client->fd );
+    ESDBG_TRACE( printf( "(%02d) locking sound daemon\n", client->fd ); );
     esd_is_locked = 1;
 
     ESD_WRITE_INT( client->fd, &client_ok, sizeof(client_ok), actual, "lock ok" );
-    fsync( client->fd );
+    /* fsync( client->fd ); */
 
     client->state = ESD_NEXT_REQUEST;
     return ok;
@@ -146,12 +147,11 @@ int esd_proto_unlock( esd_client_t *client )
     int ok = 1, actual;		/* already validated, can't fail */
     int client_ok = maybe_swap_32( client->swap_byte_order, ok );
 
-    if ( esdbg_trace )
-	printf( "(%02d) unlocking sound daemon\n", client->fd );
+    ESDBG_TRACE( printf( "(%02d) unlocking sound daemon\n", client->fd ); );
     esd_is_locked = 0;
 
     ESD_WRITE_INT( client->fd, &client_ok, sizeof(client_ok), actual, "unlck ok" );
-    fsync( client->fd );
+    /* fsync( client->fd ); */
 
     client->state = ESD_NEXT_REQUEST;
     return ok;
@@ -167,8 +167,8 @@ int esd_proto_standby( esd_client_t *client )
     /* only bother if we're not on standby */
     if ( !esd_on_standby ) {
 
-	if ( esdbg_trace )
-	    printf( "(%02d) setting sound daemon to standby\n", client->fd );
+	ESDBG_TRACE( printf( "(%02d) setting sound daemon to standby\n", 
+			     client->fd ); );
 	
 	/* TODO: close down any recorders, too */
 	esd_on_standby = 1;
@@ -176,7 +176,7 @@ int esd_proto_standby( esd_client_t *client )
     }	
 
     ESD_WRITE_INT( client->fd, &client_ok, sizeof(client_ok), actual, "stdby ok" );
-    fsync( client->fd );
+    /* fsync( client->fd ); */
 
     client->state = ESD_NEXT_REQUEST;
     return ok;
@@ -192,8 +192,7 @@ int esd_proto_resume( esd_client_t *client )
     /* only bother if we're on standby */
     if ( esd_on_standby ) {
 	
-	if ( esdbg_trace ) 
-	    printf( "(%02d) resuming sound daemon\n", client->fd );
+	ESDBG_TRACE( printf( "(%02d) resuming sound daemon\n", client->fd ); );
 	
 	/* reclaim the audio device */
 	if ( esd_audio_open() < 0 ) {
@@ -206,7 +205,7 @@ int esd_proto_resume( esd_client_t *client )
     }
 
     ESD_WRITE_INT( client->fd, &client_ok, sizeof(client_ok), actual, "resum ok" );
-    fsync( client->fd );
+    /* fsync( client->fd ); */
 
     client->state = ESD_NEXT_REQUEST;
     return ok;
@@ -254,8 +253,7 @@ int esd_proto_stream_recorder( esd_client_t *client )
 	/* flesh out the recorder */
 	esd_recorder->parent = client;
 
-	if ( esdbg_trace )
-	    printf ( "(%02d) recording on client\n", client->fd );
+	ESDBG_TRACE( printf ( "(%02d) recording on client\n", client->fd ); );
 
     } else {
 	/* failed to initialize the recorder, kill its client */
@@ -280,8 +278,7 @@ int esd_proto_stream_monitor( esd_client_t *client )
 	monitor->next = esd_monitor_list;
 	esd_monitor_list = monitor;
 
-	if ( esdbg_trace )
-	    printf ( "(%02d) monitoring on client\n", client->fd );
+	ESDBG_TRACE( printf ( "(%02d) monitoring on client\n", client->fd ); );
     } else {
 	/* failed to initialize the recorder, kill its client */
 	return 0;
@@ -305,8 +302,7 @@ int esd_proto_stream_filter( esd_client_t *client )
 	filter->next = esd_filter_list;
 	esd_filter_list = filter;
 
-	if ( esdbg_trace )
-	    printf ( "(%02d) filter on client\n", client->fd );
+	ESDBG_TRACE( printf ( "(%02d) filter on client\n", client->fd ); );
     } else {
 	/* failed to initialize the filter, kill its client */
 	return 0;
@@ -324,8 +320,7 @@ int esd_proto_sample_cache( esd_client_t *client )
     int length;
     int client_id;
 
-    if ( esdbg_trace )
-	printf( "(%02d) proto: caching sample\n", client->fd );
+    ESDBG_TRACE( printf( "(%02d) proto: caching sample\n", client->fd ); );
 
     if ( client->state == ESD_CACHING_SAMPLE ) {
 	sample = find_caching_sample( client );
@@ -339,12 +334,11 @@ int esd_proto_sample_cache( esd_client_t *client )
 	sample->parent = client;
 	if( ( length = read_sample( sample ) ) < sample->sample_length ) {
 	    client->state = ESD_CACHING_SAMPLE;
-	    if ( esdbg_trace ) 
-		printf( "(%02d) continue caching sample next trip\n", client->fd );
+	    ESDBG_TRACE( printf( "(%02d) continue caching sample next trip\n", 
+				 client->fd ); );
 	    return 1;
 	} else {
-	    if ( esdbg_trace ) 
-		printf( "(%02d) sample cached, moving on\n", client->fd );
+	    ESDBG_TRACE( printf( "(%02d) sample cached, moving on\n", client->fd ); );
 	    client->state = ESD_NEXT_REQUEST;
 	}
     } else {
@@ -356,7 +350,7 @@ int esd_proto_sample_cache( esd_client_t *client )
     client_id = maybe_swap_32( client->swap_byte_order, 
 			       sample->sample_id );
     ESD_WRITE_INT( client->fd, &client_id, sizeof(client_id), length, "smp cach" );
-    fsync( client->fd );
+    /* fsync( client->fd ); */
     return 1;
 }
 
@@ -368,8 +362,7 @@ int esd_proto_sample_getid(esd_client_t *client)
     esd_sample_t *sample = esd_samples_list;
     char namebuf[ESD_NAME_MAX];
 
-    if ( esdbg_trace )
-	printf( "(%02d) proto: getting sample ID\n", client->fd );
+    ESDBG_TRACE( printf( "(%02d) proto: getting sample ID\n", client->fd ); );
 
     ESD_READ_BIN(client->fd, namebuf, ESD_NAME_MAX, actual, "smp getid");
     namebuf[ESD_NAME_MAX - 1] = '\0';
@@ -387,7 +380,7 @@ int esd_proto_sample_getid(esd_client_t *client)
 	client_id = maybe_swap_32(client->swap_byte_order, -1);
 
     ESD_WRITE_INT( client->fd, &client_id, sizeof(client_id), length, "smp getid" );
-    fsync( client->fd );
+    /* fsync( client->fd ); */
     return 1;
 }
 
@@ -406,8 +399,8 @@ int esd_proto_sample_free( esd_client_t *client )
 
     sample_id = maybe_swap_32( client->swap_byte_order, client_id );
 
-    if ( esdbg_trace )
-	printf( "(%02d) proto: erasing sample <%d>\n", client->fd, sample_id );
+    ESDBG_TRACE( printf( "(%02d) proto: erasing sample <%d>\n", 
+			 client->fd, sample_id ); );
     erase_sample( sample_id );
 
     ESD_WRITE_INT( client->fd, &client_id, sizeof(client_id), actual, "smp free" );
@@ -431,8 +424,8 @@ int esd_proto_sample_play( esd_client_t *client )
     }
     sample_id = maybe_swap_32( client->swap_byte_order, client_id );
 
-    if ( esdbg_trace )
-	printf( "(%02d) proto: playing sample <%d>\n", client->fd, sample_id );
+    ESDBG_TRACE( printf( "(%02d) proto: playing sample <%d>\n", 
+			 client->fd, sample_id ); );
     if ( !play_sample( sample_id, 0 ) )
 	sample_id = 0;
 
@@ -458,8 +451,8 @@ int esd_proto_sample_loop( esd_client_t *client )
 
     sample_id = maybe_swap_32( client->swap_byte_order, client_id );
 
-    if ( esdbg_trace )
-	printf( "(%02d) proto: looping sample <%d>\n", client->fd, sample_id );
+    ESDBG_TRACE( printf( "(%02d) proto: looping sample <%d>\n", 
+			 client->fd, sample_id ); );
 
     if ( !play_sample( sample_id, 1 ) )
 	sample_id = 0;
@@ -486,8 +479,8 @@ int esd_proto_sample_stop( esd_client_t *client )
     
     sample_id = maybe_swap_32( client->swap_byte_order, client_id );
 
-    if ( esdbg_trace )
-	printf( "(%02d) proto: stopping sample <%d>\n", client->fd, sample_id );
+    ESDBG_TRACE( printf( "(%02d) proto: stopping sample <%d>\n", 
+			 client->fd, sample_id ); );
 
     if ( !stop_sample( sample_id ) )
 	sample_id = 0;
@@ -500,6 +493,212 @@ int esd_proto_sample_stop( esd_client_t *client )
     return 1;
 }
 
+/*******************************************************************/
+/* play a sample cached by the client, return boolean ok */
+int esd_proto_server_info( esd_client_t *client )
+{
+    int version, rate, format, actual;
+
+    version = maybe_swap_32( client->swap_byte_order, 0 );
+    rate = maybe_swap_32( client->swap_byte_order, esd_audio_rate );
+    format = maybe_swap_32( client->swap_byte_order, esd_audio_format );
+
+    ESDBG_TRACE( printf( "(%02d) proto: server info\n", client->fd ); );
+
+    /* send back the server information */
+    ESD_WRITE_INT( client->fd, &version, sizeof(version), actual, "si ver" );
+    ESD_WRITE_INT( client->fd, &rate, sizeof(rate), actual, "si rate" );
+    ESD_WRITE_INT( client->fd, &format, sizeof(format), actual, "si fmt" );
+    if ( sizeof( format ) != actual )
+	return 0;
+
+    client->state = ESD_NEXT_REQUEST;
+    return 1;
+}
+
+/*******************************************************************/
+/* play a sample cached by the client, return boolean ok */
+int esd_proto_all_info( esd_client_t *client )
+{
+    int version, rate, left, right, format;
+    int actual, source_id, sample_id, length;
+    esd_player_t *player;
+    esd_sample_t *sample;
+    const char *name;
+    char no_name[ ESD_NAME_MAX ] = "";
+
+    version = maybe_swap_32( client->swap_byte_order, 0 );
+    rate = maybe_swap_32( client->swap_byte_order, esd_audio_rate );
+    format = maybe_swap_32( client->swap_byte_order, esd_audio_format );
+
+    ESDBG_TRACE( printf( "(%02d) proto: server info\n", client->fd ); );
+
+    /* send back the server information */
+    ESD_WRITE_INT( client->fd, &version, sizeof(version), actual, "ai ver" );
+    ESD_WRITE_INT( client->fd, &rate, sizeof(rate), actual, "ai rate" );
+    ESD_WRITE_INT( client->fd, &format, sizeof(format), actual, "ai fmt" );
+    if ( sizeof( format ) != actual )
+	return 0;
+
+    /* send back the player information */
+    for ( player = esd_players_list; /* NULL will break; */ ; player = player->next )
+    {
+	if ( player ) {
+	    source_id = maybe_swap_32( client->swap_byte_order, player->source_id );
+	    name = ( (player->format & ESD_MASK_MODE) == ESD_STREAM ) 
+		? player->name : ( (esd_sample_t*) (player->parent) )->name;
+	    rate = maybe_swap_32( client->swap_byte_order, player->rate );
+	    left = maybe_swap_32( client->swap_byte_order, player->left_vol_scale );
+	    right = maybe_swap_32( client->swap_byte_order, player->right_vol_scale );
+	    format = maybe_swap_32( client->swap_byte_order, player->format );
+	} else {
+	    source_id = rate = format = 0;
+	    name = no_name;
+	}
+
+	ESD_WRITE_INT( client->fd, &source_id, sizeof(source_id), actual, "ai p.id" );
+	ESD_WRITE_BIN( client->fd, name, ESD_NAME_MAX, actual, "ai p.nm" );
+	ESD_WRITE_INT( client->fd, &rate, sizeof(rate), actual, "ai p.rate" );
+	ESD_WRITE_INT( client->fd, &left, sizeof(left), actual, "ai p.lt" );
+	ESD_WRITE_INT( client->fd, &right, sizeof(right), actual, "ai p.rt" );
+	ESD_WRITE_INT( client->fd, &format, sizeof(format), actual, "ai p.fmt" );
+	if ( sizeof( format ) != actual )
+	    return 0;
+
+	if ( !player ) break;
+    }
+
+    /* send back the sample information */
+    for ( sample = esd_samples_list; /* NULL will break; */ ; sample = sample->next )
+    {
+	if ( sample ) {
+	    sample_id = maybe_swap_32( client->swap_byte_order, sample->sample_id );
+	    name = sample->name;
+	    rate = maybe_swap_32( client->swap_byte_order, sample->rate );
+	    left = maybe_swap_32( client->swap_byte_order, sample->left_vol_scale );
+	    right = maybe_swap_32( client->swap_byte_order, sample->right_vol_scale );
+	    format = maybe_swap_32( client->swap_byte_order, sample->format );
+	    length = maybe_swap_32( client->swap_byte_order, sample->sample_length );
+	} else {
+	    sample_id = rate = format = length = 0;
+	    name = no_name;
+	}
+
+	ESD_WRITE_INT( client->fd, &sample_id, sizeof(sample_id), actual, "ai s.id" );
+	ESD_WRITE_BIN( client->fd, name, ESD_NAME_MAX, actual, "ai s.nm" );
+	ESD_WRITE_INT( client->fd, &rate, sizeof(rate), actual, "ai s.rate" );
+	ESD_WRITE_INT( client->fd, &left, sizeof(left), actual, "ai s.lt" );
+	ESD_WRITE_INT( client->fd, &right, sizeof(right), actual, "ai s.rt" );
+	ESD_WRITE_INT( client->fd, &format, sizeof(format), actual, "ai s.fmt" );
+	ESD_WRITE_INT( client->fd, &length, sizeof(length), actual, "ai s.fmt" );
+	if ( sizeof( length ) != actual )
+	    return 0;
+
+	if ( !sample ) break;
+    }
+
+    client->state = ESD_NEXT_REQUEST;
+    return 1;
+}
+
+
+/*******************************************************************/
+/* set the stereo panning for a stream */
+int esd_proto_stream_pan( esd_client_t *client )
+{
+    int client_id, client_left, client_right, client_ok, actual;
+    int stream_id, left, right, ok;
+    esd_player_t *player;
+    
+    ESD_READ_INT( client->fd, &client_id, 
+		  sizeof(client_id), actual, "panstr id" );
+    ESD_READ_INT( client->fd, &client_left, 
+		  sizeof(client_left), actual, "panstr lt" );
+    ESD_READ_INT( client->fd, &client_right, 
+		  sizeof(client_right), actual, "panstr rt" );
+    if ( sizeof( client_right ) != actual ) {
+	return 0;
+    }
+    stream_id = maybe_swap_32( client->swap_byte_order, client_id );
+    left = maybe_swap_32( client->swap_byte_order, client_left );
+    right = maybe_swap_32( client->swap_byte_order, client_right );
+
+    ESDBG_TRACE( printf( "(%02d) proto: panning stream <%d> [%d, %d]\n", 
+			 client->fd, stream_id, left, right ); );
+
+    /* find the stream, and reset panning */
+    ok = 0;
+    for ( player = esd_players_list ; player != NULL ; player = player->next )
+    {
+	if ( player->source_id == stream_id 
+	     && ( (player->format & ESD_MASK_MODE) == ESD_STREAM ) )
+	{
+	    player->left_vol_scale = left;
+	    player->right_vol_scale = right;
+	    ok = 1;
+	    break;
+	}
+    }
+
+    /* let the client know how it went */
+    client_ok = maybe_swap_32( client->swap_byte_order, ok );
+    ESD_WRITE_INT( client->fd, &client_ok, sizeof(client_ok), actual, "panstr ok" );
+    if ( sizeof( client_ok ) != actual )
+	return 0;
+
+    client->state = ESD_NEXT_REQUEST;
+    return 1;
+}
+
+
+/*******************************************************************/
+/* set the default panning for a stream */
+int esd_proto_sample_pan( esd_client_t *client )
+{
+    int client_id, client_left, client_right, client_ok, actual;
+    int sample_id, left, right, ok;
+    esd_sample_t *sample;
+    
+    ESD_READ_INT( client->fd, &client_id, 
+		  sizeof(client_id), actual, "pansam id" );
+    ESD_READ_INT( client->fd, &client_left, 
+		  sizeof(client_left), actual, "pansam lt" );
+    ESD_READ_INT( client->fd, &client_right, 
+		  sizeof(client_right), actual, "pansam rt" );
+    if ( sizeof( client_right ) != actual ) {
+	return 0;
+    }
+    sample_id = maybe_swap_32( client->swap_byte_order, client_id );
+    left = maybe_swap_32( client->swap_byte_order, client_left );
+    right = maybe_swap_32( client->swap_byte_order, client_right );
+
+    ESDBG_TRACE( printf( "(%02d) proto: panning sample <%d> [%d, %d]\n", 
+			 client->fd, sample_id, left, right ); );
+
+    /* find the stream, and reset panning */
+    ok = 0;
+    for ( sample = esd_samples_list ; sample != NULL ; sample = sample->next )
+    {
+	if ( sample->sample_id == sample_id )
+	{
+	    sample->left_vol_scale = left;
+	    sample->right_vol_scale = right;
+	    ok = 1;
+	    break;
+	}
+    }
+
+    /* let the client know how it went */
+    client_ok = maybe_swap_32( client->swap_byte_order, ok );
+    ESD_WRITE_INT( client->fd, &client_ok, sizeof(client_ok), actual, "panstr ok" );
+    if ( sizeof( client_ok ) != actual )
+	return 0;
+
+    client->state = ESD_NEXT_REQUEST;
+    return 1;
+}
+
+
 /*******************************************************************/
 /* now that we trust the client, do it's bidding, return boolean ok */
 static int do_validated_action ( esd_client_t *client )
@@ -586,8 +785,8 @@ int poll_client_requests()
  	    }
  	    else if ( client->state == ESD_NEEDS_ENDCHECK ) {
 		/* wait for next trip through */
-		if ( esdbg_trace )
-		    printf( "waiting to check for endian key next trip\n" );
+		ESDBG_TRACE( 
+		    printf( "waiting to check for endian key next trip\n" ); );
 	    } 
  	    else if ( client->request != ESD_PROTO_INVALID ) {
  	        /* send failure code to client */
@@ -595,11 +794,11 @@ int poll_client_requests()
 			 "(%02d) poll: only owner may control sound daemon\n",
 			 client->fd );
 		ESD_WRITE_INT( client->fd, &is_ok, sizeof(is_ok), length, "poll val" );
-		fsync( client->fd );
+		/* fsync( client->fd ); */
  	    }
  	}
 	else if ( client->state == ESD_NEEDS_ENDCHECK ) {
-	    if ( esdbg_trace ) printf( "rechecking for endian key\n" );
+	    ESDBG_TRACE( printf( "rechecking for endian key\n" ); );
 	    is_ok = check_endian( client );
 
 	    /* check_endian() return vals don't exactly match is_ok's meaning */
@@ -608,11 +807,11 @@ int poll_client_requests()
 	    else client->state = ESD_NEXT_REQUEST;
 	}
 	else if ( client->state == ESD_CACHING_SAMPLE ) {
-	    if ( esdbg_trace ) printf( "resuming sample cache\n" );
+	    ESDBG_TRACE( printf( "resuming sample cache\n" ); );
 	    is_ok = esd_proto_sample_cache( client );
 	}
 	else if ( client->state == ESD_NEEDS_REQDATA ) {
-	    if ( esdbg_trace ) printf( "resuming request data\n" );
+	    ESDBG_TRACE( printf( "resuming request data\n" ); );
 
 	    switch ( client->request )
 	    {
@@ -629,8 +828,8 @@ int poll_client_requests()
 		is_ok = esd_proto_sample_stop( client ); break;
 
 	    default:
-		if ( esdbg_trace ) printf( "(%02d) invalid request: %d\n",
-					   client->fd, client->request );
+		ESDBG_TRACE( printf( "(%02d) invalid request: %d\n",
+				     client->fd, client->request ); );
 		break;
 	    }
 	    
@@ -638,18 +837,19 @@ int poll_client_requests()
  	else {
  
  	    /* make sure there's a request as EOF may return as readable */
-	    if ( esdbg_comms ) printf( "--------------------------------\n" );
+	    /* if ( esdbg_comms ) printf( "--------------------------------\n" ); */
+	    ESDBG_COMMS( printf( "--------------------------------\n" ); );
 	    ESD_READ_INT( client->fd, &client->request, sizeof(client->request), 
 			  length, "request" );
-	    if ( esdbg_comms ) printf( "\n" );
+	    /* if ( esdbg_comms ) printf( "\n" ); */
+	    ESDBG_COMMS( printf( "\n" ); );
 	    if ( client->swap_byte_order )
 		client->request = swap_endian_32( client->request );
 
  	    if ( length <= 0 ) {
  		/* no more data available from that client, close it */
-		if ( esdbg_trace )
-		    printf( "(%02d) no more protocol requests for client\n", 
-			    client->fd );
+		ESDBG_TRACE( printf( "(%02d) no more protocol requests for client\n", 
+				     client->fd ); );
  		is_ok = 0;
  
  	    } else {
@@ -694,19 +894,43 @@ int poll_client_requests()
  		case ESD_PROTO_SAMPLE_FREE:
  		    is_ok = esd_proto_sample_free( client );
  		    break;
- 
+
  		case ESD_PROTO_SAMPLE_PLAY:
  		    is_ok = esd_proto_sample_play( client );
  		    break;
- 
+
  		case ESD_PROTO_SAMPLE_LOOP:
  		    is_ok = esd_proto_sample_loop( client );
  		    break;
- 
+
  		case ESD_PROTO_SAMPLE_STOP:
  		    is_ok = esd_proto_sample_stop( client );
  		    break;
- 
+
+		case ESD_PROTO_SERVER_INFO:
+ 		    is_ok = esd_proto_server_info( client );
+ 		    break;
+
+		case ESD_PROTO_ALL_INFO:
+ 		    is_ok = esd_proto_all_info( client );
+ 		    break;
+
+		case ESD_PROTO_SUBSCRIBE:
+		case ESD_PROTO_UNSUBSCRIBE:
+		    fprintf( stderr, 
+			     "(%02d) proto: unimplemented protocol request:  0x%08x\n",
+			     client->fd, client->request );
+		    is_ok = 0;
+		    break;
+
+		case ESD_PROTO_STREAM_PAN:
+ 		    is_ok = esd_proto_stream_pan( client );
+		    break;
+
+		case ESD_PROTO_SAMPLE_PAN:
+ 		    is_ok = esd_proto_sample_pan( client );
+		    break;
+
  		default:
 		    fprintf( stderr, 
 			     "(%02d) proto: unknown protocol request:  0x%08x\n",
